=== modified file 'loggerhead/apps/filesystem.py'
--- loggerhead/apps/filesystem.py	2008-07-26 18:39:49 +0000
+++ loggerhead/apps/filesystem.py	2008-08-01 04:18:27 +0000
@@ -36,15 +36,16 @@
 
 
 class BranchesFromFileSystemServer(object):
-    def __init__(self, folder, root):
-        self.folder = folder
+    def __init__(self, path, root, name=None):
+        self.path = path
         self.root = root
+        self.name = name
 
     def app_for_branch(self, branch):
-        if not self.folder:
-            name = os.path.basename(os.path.abspath(self.root.folder))
+        if not self.name:
+            name = branch.nick
         else:
-            name = self.folder
+            name = self.name
         branch_app = BranchWSGIApp(
             branch, name, {'cachepath': sql_dir}, self.root.graph_cache)
         return branch_app.app
@@ -55,17 +56,20 @@
             raise httpexceptions.HTTPMovedPermanently(
                 environ['SCRIPT_NAME'] + '/')
         elif segment == '':
-            return DirectoryListing(os.path.join(self.root.folder, self.folder))
+            return DirectoryListing(self.path)
         else:
-            relpath = os.path.join(self.folder, segment)
-            return BranchesFromFileSystemServer(relpath, self.root)
+            new_path = os.path.join(self.path, segment)
+            if self.name:
+                new_name = os.path.join(self.name, segment)
+            else:
+                new_name = segment
+            return BranchesFromFileSystemServer(new_path, self.root, new_name)
 
     def __call__(self, environ, start_response):
-        path = os.path.join(self.root.folder, self.folder)
-        if not os.path.isdir(path):
+        if not os.path.isdir(self.path):
             raise httpexceptions.HTTPNotFound()
         try:
-            b = branch.Branch.open(path)
+            b = branch.Branch.open(self.path)
         except errors.NotBranchError:
             return self.app_for_non_branch(environ)(environ, start_response)
         else:
@@ -88,4 +92,33 @@
             return favicon_app(environ, start_response)
         else:
             return BranchesFromFileSystemServer(
-                '', self)(environ, start_response)
+                self.folder, self)(environ, start_response)
+
+
+class UserBranchesFromFileSystemRoot(object):
+
+    def __init__(self, folder, trunk_dir):
+        self.graph_cache = lru_cache.LRUCache()
+        self.folder = folder
+        self.trunk_dir = trunk_dir
+
+    def __call__(self, environ, start_response):
+        environ['loggerhead.static.url'] = environ['SCRIPT_NAME']
+        path_info= environ['PATH_INFO']
+        if path_info.startswith('/static/'):
+            segment = path_info_pop(environ)
+            assert segment == 'static'
+            return static_app(environ, start_response)
+        elif path_info == '/favicon.ico':
+            return favicon_app(environ, start_response)
+        else:
+            # segments starting with ~ are user branches
+            if path_info.startswith('/~'):
+                segment = path_info_pop(environ)
+                new_path = os.path.join(self.folder, segment[1:])
+                return BranchesFromFileSystemServer(
+                    new_path, self, segment)(environ, start_response)
+            else:
+                new_path = os.path.join(self.folder, self.trunk_dir)
+                return BranchesFromFileSystemServer(
+                    new_path, self)(environ, start_response)

=== modified file 'serve-branches'
--- serve-branches	2008-07-26 17:41:49 +0000
+++ serve-branches	2008-08-01 04:42:14 +0000
@@ -16,36 +16,81 @@
 """Search for branches underneath a directory and serve them all."""
 
 import logging
+import os
 import sys
 
+from optparse import OptionParser
+
 from paste import httpserver
 from paste.httpexceptions import HTTPExceptionHandler
 from paste.translogger import TransLogger
 
-from loggerhead.apps.filesystem import BranchesFromFileSystemRoot
-
-
-logging.basicConfig()
-logging.getLogger().setLevel(logging.DEBUG)
-
-if len(sys.argv) > 1:
-    path = sys.argv[1]
-else:
-    path = '.'
-
-app = BranchesFromFileSystemRoot(path)
-
-app = HTTPExceptionHandler(app)
-app = TransLogger(app)
-
-try:
-    from paste.deploy.config import PrefixMiddleware
-except ImportError:
-    pass
-else:
-    app = PrefixMiddleware(app)
-
-#from paste.evalexception import EvalException
-#app = EvalException(app)
-
-httpserver.serve(app, host='0.0.0.0', port='8080')
+from loggerhead import __version__
+from loggerhead.apps.filesystem import (
+    BranchesFromFileSystemRoot, UserBranchesFromFileSystemRoot)
+
+
+def command_line_parser():
+    parser = OptionParser("%prog [options] <path>")
+    parser.set_defaults(
+        user_dirs=False,
+        show_version=False,
+        )
+    parser.add_option("--user-dirs", action="store_true", dest="user_dirs",
+                      help="Serve user directories as ~user.")
+    parser.add_option("--trunk-dir", metavar="DIR",
+                      help="The directory that contains the trunk branches.")
+    parser.add_option("--version", action="store_true", dest="show_version",
+                      help="Print the software version and exit")
+    return parser
+
+
+def main(args):
+    logging.basicConfig()
+    logging.getLogger().setLevel(logging.DEBUG)
+
+    parser = command_line_parser()
+    (options, args) = parser.parse_args(sys.argv[1:])
+
+    if options.show_version:
+        print "loggerhead %s" % __version__
+        sys.exit(0)
+
+    if len(args) > 1:
+        parser.print_help()
+        sys.exit(1)
+    elif len(args) == 1:
+        [path] = args
+    else:
+        path = '.'
+
+    if not os.path.isdir(path):
+        print "%s is not a directory" % path
+        sys.exit(1)
+
+    if options.trunk_dir and not options.user_dirs:
+        print "--trunk-dir is only valid with --user-dirs"
+        sys.exit(1)
+
+    if options.user_dirs:
+        if not options.trunk_dir:
+            print "You didn't specify a directory for the trunk directories."
+            sys.exit(1)
+        app = UserBranchesFromFileSystemRoot(path, options.trunk_dir)
+    else:
+        app = BranchesFromFileSystemRoot(path)
+    app = HTTPExceptionHandler(app)
+    app = TransLogger(app)
+
+    try:
+        from paste.deploy.config import PrefixMiddleware
+    except ImportError:
+        pass
+    else:
+        app = PrefixMiddleware(app)
+
+    httpserver.serve(app, host='0.0.0.0', port='8080')
+
+
+if __name__ == "__main__":
+    main(sys.argv)

